@@ -407,14 +407,10 @@ bgp_damp_parameter_set (int hlife, int reuse, int sup, int maxsup)
   damp->reuse_list = XCALLOC (MTYPE_BGP_DAMP_ARRAY, 
 			      damp->reuse_list_size 
 			      * sizeof (struct bgp_reuse_node *));
-  memset (damp->reuse_list, 0x00, 
-          damp->reuse_list_size * sizeof (struct bgp_reuse_node *));  
 
   /* Reuse-array computations */
-  damp->reuse_index = XMALLOC (MTYPE_BGP_DAMP_ARRAY, 
+  damp->reuse_index = XCALLOC (MTYPE_BGP_DAMP_ARRAY,
 			       sizeof(int) * damp->reuse_index_size);
-  memset (damp->reuse_index, 0x00,
-          damp->reuse_list_size * sizeof (int));
 
   reuse_max_ratio = (double)damp->ceiling/damp->reuse_limit;
   j = (exp((double)damp->max_suppress_time/damp->half_life) * log10(2.0));
@@ -502,6 +498,10 @@ bgp_damp_info_clean (void)
 int
 bgp_damp_disable (struct bgp *bgp, afi_t afi, safi_t safi)
 {
+  /* If it wasn't enabled, there's nothing to do. */
+  if (! CHECK_FLAG (bgp->af_flags[afi][safi], BGP_CONFIG_DAMPENING))
+    return 0;
+
   /* Cancel reuse thread. */
   if (damp->t_reuse )
     thread_cancel (damp->t_reuse);
