@@ -355,7 +355,7 @@ bgp_graceful_restart_timer_expire (struct thread *thread)
 
   /* NSF delete stale route */
   for (afi = AFI_IP ; afi < AFI_MAX ; afi++)
-    for (safi = SAFI_UNICAST ; safi < SAFI_UNICAST_MULTICAST ; safi++)
+    for (safi = SAFI_UNICAST ; safi < SAFI_RESERVED_3 ; safi++)
       if (peer->nsf[afi][safi])
 	bgp_clear_stale_route (peer, afi, safi);
 
@@ -388,7 +388,7 @@ bgp_graceful_stale_timer_expire (struct thread *thread)
 
   /* NSF delete stale route */
   for (afi = AFI_IP ; afi < AFI_MAX ; afi++)
-    for (safi = SAFI_UNICAST ; safi < SAFI_UNICAST_MULTICAST ; safi++)
+    for (safi = SAFI_UNICAST ; safi < SAFI_RESERVED_3 ; safi++)
       if (peer->nsf[afi][safi])
 	bgp_clear_stale_route (peer, afi, safi);
 
@@ -481,7 +481,7 @@ bgp_stop (struct peer *peer)
 	  UNSET_FLAG (peer->sflags, PEER_STATUS_NSF_MODE);
 
 	  for (afi = AFI_IP ; afi < AFI_MAX ; afi++)
-	    for (safi = SAFI_UNICAST ; safi < SAFI_UNICAST_MULTICAST ; safi++)
+	    for (safi = SAFI_UNICAST ; safi < SAFI_RESERVED_3 ; safi++)
 	      peer->nsf[afi][safi] = 0;
 	}
 
@@ -597,8 +597,6 @@ bgp_stop_with_error (struct peer *peer)
 static int
 bgp_connect_success (struct peer *peer)
 {
-  char buf1[BUFSIZ];
-
   if (peer->fd < 0)
     {
       zlog_err ("bgp_connect_success peer's fd is negative value %d",
@@ -612,6 +610,8 @@ bgp_connect_success (struct peer *peer)
 
   if (BGP_DEBUG (normal, NORMAL))
     {
+      char buf1[SU_ADDRSTRLEN];
+
       if (! CHECK_FLAG (peer->sflags, PEER_STATUS_ACCEPT_PEER))
 	zlog_debug ("%s open active, local address %s", peer->host,
 		    sockunion2str (peer->su_local, buf1, SU_ADDRSTRLEN));
@@ -799,7 +799,7 @@ bgp_establish (struct peer *peer)
   /* graceful restart */
   UNSET_FLAG (peer->sflags, PEER_STATUS_NSF_WAIT);
   for (afi = AFI_IP ; afi < AFI_MAX ; afi++)
-    for (safi = SAFI_UNICAST ; safi < SAFI_UNICAST_MULTICAST ; safi++)
+    for (safi = SAFI_UNICAST ; safi < SAFI_RESERVED_3 ; safi++)
       {
 	if (peer->afc_nego[afi][safi]
 	    && CHECK_FLAG (peer->cap, PEER_CAP_RESTART_ADV)
