@@ -92,7 +92,7 @@ DEFUN (show_version_ospf6,
   return CMD_SUCCESS;
 }
 
-struct cmd_node debug_node =
+static struct cmd_node debug_node =
 {
   DEBUG_NODE,
   "",
@@ -1647,6 +1647,12 @@ DEFUN (show_ipv6_ospf6_border_routers,
 
       ospf6_linkstate_prefix (adv_router, 0, &prefix);
       ro = ospf6_route_lookup (&prefix, ospf6->brouter_table);
+      if (!ro)
+        {
+          vty_out (vty, "No Route found for Router ID: %s%s", argv[0], VNL);
+          return CMD_SUCCESS;
+        }
+
       ospf6_route_show_detail (vty, ro);
       return CMD_SUCCESS;
     }
@@ -1883,4 +1889,11 @@ ospf6_init (void)
   thread_add_read (master, ospf6_receive, NULL, ospf6_sock);
 }
 
-
+void
+ospf6_clean (void)
+{
+  if (ospf6->route_table)
+    ospf6_route_remove_all (ospf6->route_table);
+  if (ospf6->brouter_table)
+    ospf6_route_remove_all (ospf6->brouter_table);
+}
