@@ -141,7 +141,7 @@ extern void ospf_intra_add_transit (struct route_table *, struct vertex *,
 extern void ospf_intra_add_stub (struct route_table *,
 				 struct router_lsa_link *, struct vertex *,
 				 struct ospf_area *,
-				 int parent_is_root);
+				 int parent_is_root, int);
 
 extern int ospf_route_cmp (struct ospf *, struct ospf_route *,
 			   struct ospf_route *);
@@ -159,7 +159,7 @@ extern void ospf_prune_unreachable_networks (struct route_table *);
 extern void ospf_prune_unreachable_routers (struct route_table *);
 extern int ospf_add_discard_route (struct route_table *, struct ospf_area *,
 				   struct prefix_ipv4 *);
-extern void ospf_delete_discard_route (struct prefix_ipv4 *);
+extern void ospf_delete_discard_route (struct route_table *, struct prefix_ipv4 *);
 extern int ospf_route_match_same (struct route_table *, struct prefix_ipv4 *,
 				  struct ospf_route *);
 
