@@ -133,7 +133,7 @@ static void ospf_mpls_te_config_write_router (struct vty *vty);
 static void ospf_mpls_te_config_write_if (struct vty *vty, struct interface *ifp);
 static void ospf_mpls_te_show_info (struct vty *vty, struct ospf_lsa *lsa);
 static int ospf_mpls_te_lsa_originate (void *arg);
-static void ospf_mpls_te_lsa_refresh (struct ospf_lsa *lsa);
+static struct ospf_lsa *ospf_mpls_te_lsa_refresh (struct ospf_lsa *lsa);
 static void ospf_mpls_te_lsa_schedule (struct mpls_te_link *lp, enum sched_opcode);
 
 static void del_mpls_te_link (void *val);
@@ -205,7 +205,7 @@ get_mpls_te_instance_value (void)
 {
   static u_int32_t seqno = 0;
 
-  if (LEGAL_TE_INSTANCE_RANGE (seqno + 1))
+  if (seqno < MAX_LEGAL_TE_INSTANCE_NUM )
     seqno += 1;
   else
     seqno  = 1; /* Avoid zero. */
@@ -561,13 +561,13 @@ ospf_mpls_te_new_if (struct interface *ifp)
       goto out;
     }
 
-  if ((new = XMALLOC (MTYPE_OSPF_MPLS_TE_LINKPARAMS,
-                  sizeof (struct mpls_te_link))) == NULL)
+  new = XCALLOC (MTYPE_OSPF_MPLS_TE_LINKPARAMS,
+                  sizeof (struct mpls_te_link));
+  if (new == NULL)
     {
       zlog_warn ("ospf_mpls_te_new_if: XMALLOC: %s", safe_strerror (errno));
       goto out;
     }
-  memset (new, 0, sizeof (struct mpls_te_link));
 
   new->area = NULL;
   new->flags = 0;
@@ -1009,7 +1009,7 @@ out:
   return rc;
 }
 
-static void
+static struct ospf_lsa *
 ospf_mpls_te_lsa_refresh (struct ospf_lsa *lsa)
 {
   struct mpls_te_link *lp;
@@ -1070,7 +1070,7 @@ ospf_mpls_te_lsa_refresh (struct ospf_lsa *lsa)
     }
 
 out:
-  return;
+  return new;
 }
 
 static void
