@@ -22,14 +22,8 @@
 #include <zebra.h>
 
 #ifdef HAVE_SNMP
-#ifdef HAVE_NETSNMP
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
-#else
-#include <asn1.h>
-#include <snmp.h>
-#include <snmp_impl.h>
-#endif
 
 #include "if.h"
 #include "log.h"
@@ -93,10 +87,14 @@ oid rip_oid [] = { RIPV2MIB };
 struct route_table *rip_ifaddr_table;
 
 /* Hook functions. */
-static u_char *rip2Globals ();
-static u_char *rip2IfStatEntry ();
-static u_char *rip2IfConfAddress ();
-static u_char *rip2PeerTable ();
+static u_char *rip2Globals (struct variable *, oid [], size_t *,
+			    int, size_t *, WriteMethod **);
+static u_char *rip2IfStatEntry (struct variable *, oid [], size_t *,
+				int, size_t *, WriteMethod **);
+static u_char *rip2IfConfAddress (struct variable *, oid [], size_t *,
+				  int, size_t *, WriteMethod **);
+static u_char *rip2PeerTable (struct variable *, oid [], size_t *,
+			      int, size_t *, WriteMethod **);
 
 struct variable rip_variables[] = 
 {
@@ -215,7 +213,7 @@ rip_ifaddr_delete (struct interface *ifp, struct connected *ifc)
     }
 }
 
-struct interface *
+static struct interface *
 rip_ifaddr_lookup_next (struct in_addr *addr)
 {
   struct prefix_ipv4 p;
@@ -347,6 +345,10 @@ rip2IfStatEntry (struct variable *v, oid name[], size_t *length,
   static struct in_addr addr;
   static long valid = SNMP_VALID;
 
+  if (smux_header_table(v, name, length, exact, var_len, write_method)
+      == MATCH_FAILED)
+    return NULL;
+
   memset (&addr, 0, sizeof (struct in_addr));
   
   /* Lookup interface. */
@@ -450,6 +452,10 @@ rip2IfConfAddress (struct variable *v, oid name[], size_t *length,
   struct interface *ifp;
   struct rip_interface *ri;
 
+  if (smux_header_table(v, name, length, exact, val_len, write_method)
+      == MATCH_FAILED)
+    return NULL;
+
   memset (&addr, 0, sizeof (struct in_addr));
   
   /* Lookup interface. */
@@ -520,6 +526,10 @@ rip2PeerTable (struct variable *v, oid name[], size_t *length,
 
   struct rip_peer *peer;
 
+  if (smux_header_table(v, name, length, exact, val_len, write_method)
+      == MATCH_FAILED)
+    return NULL;
+
   memset (&addr, 0, sizeof (struct in_addr));
   
   /* Lookup interface. */
