@@ -1,6 +1,7 @@
-
 #include <zebra.h>
 #include <sigevent.h>
+#include "lib/log.h"
+#include "lib/memory.h"
 
 void
 sighup (void)
@@ -43,8 +44,14 @@ int
 main (void)
 {
   master = thread_master_create ();
-  signal_init (master, Q_SIGC(sigs), sigs);
-
+  signal_init (master, array_size(sigs), sigs);
+  
+  zlog_default = openzlog("testsig", ZLOG_NONE,
+                          LOG_CONS|LOG_NDELAY|LOG_PID, LOG_DAEMON);
+  zlog_set_level (NULL, ZLOG_DEST_SYSLOG, ZLOG_DISABLED);
+  zlog_set_level (NULL, ZLOG_DEST_STDOUT, LOG_DEBUG);
+  zlog_set_level (NULL, ZLOG_DEST_MONITOR, ZLOG_DISABLED);
+  
   while (thread_fetch (master, &t))
     thread_call (&t);
 
