@@ -80,6 +80,7 @@ struct zebra_t
 
   /* rib work queue */
   struct work_queue *ribq;
+  struct meta_queue *mq;
 };
 
 /* Count prefix size from mask length */
@@ -88,6 +89,7 @@ struct zebra_t
 /* Prototypes. */
 extern void zebra_init (void);
 extern void zebra_if_init (void);
+extern void zebra_zserv_socket_init (char *path);
 extern void hostinfo_get (void);
 extern void rib_init (void);
 extern void interface_list (void);
@@ -107,6 +109,5 @@ extern int zsend_route_multipath (int, struct zserv *, struct prefix *,
 extern int zsend_router_id_update(struct zserv *, struct prefix *);
 
 extern pid_t pid;
-extern pid_t old_pid;
 
 #endif /* _ZEBRA_ZEBRA_H */
